<?php

require __DIR__ . '/vendor/autoload.php';

use AlibabaCloud\Client\AlibabaCloud;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;
use OSS\OssClient;
use RingCentral\Psr7\Response;

function handler($request, $context): Response {
    // 如果是 GET 的话直接返回前端文件，这个需要自定义域名支持。
    if($request->getMethod() === 'GET') {
        return new Response(
            200,
            [
                'Content-type' => 'text/html; charset=utf-8',
            ],
            file_get_contents(__DIR__ . '/index.html')
        );
    }

    // 获取存储桶名称
    $bucket = getenv('oss_bucket');
    // 获取存储桶地域
    $region = getenv('oss_region');

    // 实例化 OSS
    // $ossClient = new OssClient(
    //     'yourAccessKeyId',
    //     'yourAccessKeySecret',
    //     'https://oss-' . $region . '.aliyuncs.com'
    // );
    $ossClient = new OssClient(
        $context['credentials']['accessKeyId'],
        $context['credentials']['accessKeySecret'],
        'https://oss-' . $region . '.aliyuncs.com',
        false,
        $context['credentials']['securityToken']
    );

    // 实例化 GuzzleHttp
    // 具体接口地址参考：https://help.aliyun.com/document_detail/94737.html#h2-u670Du52A1u5730u57403
    $httpClient = new Client([
        'base_uri' => 'https://nls-gateway.cn-shanghai.aliyuncs.com/stream/v1/tts',
    ]);

    // Token 获取
    // 文档：https://help.aliyun.com/document_detail/72153.html
    // AlibabaCloud::accessKeyClient(
    //     'yourAccessKeyId',
    //     'yourAccessKeySecret')
    //     ->regionId('cn-shanghai')
    //     ->asDefaultClient();
    AlibabaCloud::StsClient(
        $context['credentials']['accessKeyId'],
        $context['credentials']['accessKeySecret'],
        $context['credentials']['securityToken'])
        ->regionId('cn-shanghai')
        ->asDefaultClient();

    try {
        // 配置参考：https://help.aliyun.com/document_detail/72153.html#h2--commonrequest-5
        $response = AlibabaCloud::rpc()
                        ->regionId('cn-shanghai')
                        ->version('2019-02-28')
                        ->action('CreateToken')
                        ->host('nls-meta.cn-shanghai.aliyuncs.com')
                        ->method('POST')
                        ->request();

        $token = $response['Token'];

        if($token == null) {
            return error('Token获取失败');
        }
    } catch (\Exception $e) {
        return error('系统错误');
    }

    // 用户输入配置
    $body = (array) json_decode($request->getBody()->getContents());

    // 对用户输入配置进行 json 编码然后 md5 加密，有效避免重复生成
    $filename = md5(json_encode($body));

    // 文件存储路径
    $filepath = sprintf('tts/%s.%s', $filename, $body['format']);

    // 获取文件是否存在
    $exist = $ossClient->doesObjectExist($bucket, $filepath);

    // 文件存在直接返回文件地址，不用重新生成
    if($exist === true) {
        return result([
            'url' => $ossClient->signUrl($bucket, $filepath, 10 * 60),
        ]);
    }

    try {
        // 调用语音合成接口
        $response = $httpClient->post('', [
            'body' => json_encode(array_merge([
                'appkey'    => getenv('appkey'),
                'token'     => $token['Id'],
            ], $body)),
        ]);
    } catch (RequestException $e) {
        // 请求失败后返回失败信息
        $body = (array) json_decode($e->getResponse()->getBody()->getContents());

        return error('[' . $body['status'] . '] ' . $body['message']);
    }

    // 将文件流上传到 OSS
    $ossClient->putObject($bucket, $filepath, $response->getBody()->getContents());

    return result([
        'url' => $ossClient->signUrl($bucket, $filepath, 10 * 60),
    ]);
}

/**
 * 返回成功
 *
 * @param array  $data  数据
 * @return RingCentral\Psr7\Response
 */
function result($data = []): Response {
    return new Response(
        200,
        [],
        json_encode([
            'code'  => 1,
            'data'  => $data,
        ])
    );
}

/**
 * 返回错误
 *
 * @param string  $msg  错误信息
 * @return RingCentral\Psr7\Response
 */
function error($msg): Response {
    return new Response(
        200,
        [],
        json_encode([
            'code'  => 0,
            'msg'   => $msg,
        ])
    );
}